@@ -7,22 +7,26 @@ use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
 @ISA = qw(Exporter);
 @EXPORT_OK   = qw(_COUNT _MULTI _COUNTMULTI _GEN_ID
 		  _HYPER _UNORD _UNIQ _REF _UNORDUNIQ _UNIONFIND _LIGHT
+		  _STR _REFSTR
 		  _n _f _a _i _s _p _g _u _ni _nc _na _nm);
 %EXPORT_TAGS =
     (flags =>  [qw(_COUNT _MULTI _COUNTMULTI _GEN_ID
-		   _HYPER _UNORD _UNIQ _REF _UNORDUNIQ _UNIONFIND _LIGHT)],
+		   _HYPER _UNORD _UNIQ _REF _UNORDUNIQ _UNIONFIND _LIGHT
+		   _STR _REFSTR)],
      fields => [qw(_n _f _a _i _s _p _g _u _ni _nc _na _nm)]);
 
-sub _COUNT      () {  0x00000001   }
-sub _MULTI      () {  0x00000002   }
-sub _COUNTMULTI () { _COUNT|_MULTI }
-sub _HYPER      () {  0x00000004   }
-sub _UNORD      () {  0x00000008   }
-sub _UNIQ       () {  0x00000010   }
-sub _REF        () {  0x00000020   }
-sub _UNORDUNIQ  () { _UNORD|_UNIQ  }
-sub _UNIONFIND  () {  0x00000040   }
-sub _LIGHT      () {  0x00000080   }
+sub _COUNT       () {  0x00000001   }
+sub _MULTI       () {  0x00000002   }
+sub _COUNTMULTI  () { _COUNT|_MULTI }
+sub _HYPER       () {  0x00000004   }
+sub _UNORD       () {  0x00000008   }
+sub _UNIQ        () {  0x00000010   }
+sub _REF         () {  0x00000020   }
+sub _UNORDUNIQ   () { _UNORD|_UNIQ  }
+sub _UNIONFIND   () {  0x00000040   }
+sub _LIGHT       () {  0x00000080   }
+sub _STR         () {  0x00000100   }
+sub _REFSTR      () { _REF|_STR     }
 
 my $_GEN_ID = 0;
 
@@ -60,6 +64,7 @@ sub has_paths {
 }
 
 sub _dump {
+    require Data::Dumper;
     my $d = Data::Dumper->new([$_[0]],[ref $_[0]]);
     defined wantarray ? $d->Dump : print $d->Dump;
 }
@@ -105,8 +110,8 @@ sub __get_path_node {
     my $m = shift;
     my ($p, $k);
     my $f = $m->[ _f ];
+    @_ = sort @_ if ($f & _UNORD);
     if ($m->[ _a ] == 2 && @_ == 2 && !($f & (_HYPER|_REF|_UNIQ))) { # Fast path.
-	@_ = sort @_ if ($f & _UNORD);
 	return unless exists $m->[ _s ]->{ $_[0] };
 	$p = [ $m->[ _s ], $m->[ _s ]->{ $_[0] } ];
 	$k = [ $_[0], $_[1] ];
@@ -138,6 +143,7 @@ sub _has_path_attrs {
     my $m = shift;
     my $f = $m->[ _f ];
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -156,9 +162,9 @@ sub _set_path_attrs {
     my $f = $m->[ _f ];
     my $attr = pop;
     my $id   = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( @_ );
     push @_, $id if ($f & _MULTI);
-    @_ = sort @_ if ($f & _UNORD);
     my ($p, $k) = $m->__set_path( @_ );
     return unless defined $p && defined $k;
     my $l = defined $k->[-1] ? $k->[-1] : "";
@@ -177,6 +183,7 @@ sub _has_path_attr {
     my $f = $m->[ _f ];
     my $attr = pop;
     my $id   = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -196,6 +203,7 @@ sub _set_path_attr {
     my $val  = pop;
     my $attr = pop;
     my $id   = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     my ($p, $k);
     $m->__attr( \@_ ); # _LIGHT maps need this to get upgraded when needed.
     push @_, $id if ($f & _MULTI);
@@ -224,6 +232,7 @@ sub _get_path_attrs {
     my $m = shift;
     my $f = $m->[ _f ];
     my $id   = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -243,6 +252,7 @@ sub _get_path_attr {
     my $f = $m->[ _f ];
     my $attr = pop;
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -260,6 +270,7 @@ sub _get_path_attr_names {
     my $m = shift;
     my $f = $m->[ _f ];
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -278,6 +289,7 @@ sub _get_path_attr_values {
     my $m = shift;
     my $f = $m->[ _f ];
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -296,6 +308,7 @@ sub _del_path_attrs {
     my $m = shift;
     my $f = $m->[ _f ];
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -325,6 +338,7 @@ sub _del_path_attr {
     my $f = $m->[ _f ];
     my $attr = pop;
     my $id = pop if ($f & _MULTI);
+    @_ = sort @_ if ($f & _UNORD);
     $m->__attr( \@_ );
     if (($f & _MULTI)) {
 	my ($p, $k) = $m->__has_path( @_ );
@@ -351,6 +365,7 @@ sub _is_HYPER { $_[0]->[ _f ] & _HYPER }
 sub _is_UNORD { $_[0]->[ _f ] & _UNORD }
 sub _is_UNIQ  { $_[0]->[ _f ] & _UNIQ  }
 sub _is_REF   { $_[0]->[ _f ] & _REF   }
+sub _is_STR   { $_[0]->[ _f ] & _STR   }
 
 sub __arg {
     my $m = shift;
@@ -437,7 +452,7 @@ Return true if the Map has any paths, false if not.
 
 =item has_path_by_multi_id($id)
 
-Return true ifd the a Map has the path by a multi(vertex) id, false if not.
+Return true if the Map has the path by a multi(vertex) id, false if not.
 
 =item paths
 
